module Domotics::Core
  class Switch < Element
    MINIMUM_LAG = 0
    def initialize(args = {})
      @type = args[:type] || :switch
      @lag = nil
      @lag_lock = Mutex.new
      args[:driver] = "DigitalPin"
      load_driver args
      @initialized = false
      super
      @initialized = true
    end

    def set_state(value)
      @initialized ? (super unless state == value) : super
    end

    def on(timer = nil)
      set_state :on
      lag(:off, timer)
    end

    def on?
      state == :on
    end

    def delay_on(timer)
      lag(:on, timer)
    end

    def off(timer = nil)
      set_state :off
      lag(:on, timer)
    end

    def off?
      state == :off
    end

    def delay_off(timer, &block)
      lag(:off, timer, &block)
    end

    def toggle(timer = nil)
      set_state state == :off ? :on : :off
      lag(:toggle, timer)
    end

    def delay_toggle(timer)
      lag(:toggle, timer)
    end

    private

    def lag(action = nil, timer = nil, &block)
      # Kill previous action -> out of date
      @lag_lock.synchronize do
        if @lag && @lag.alive?
          @lag.kill
          @lag = nil
        end
        raise ArgumentError unless timer.is_a?(Integer) && (timer >= MINIMUM_LAG)

        # Delayed action
        @lag = Thread.new {
          sleep timer
          block_given? ? (public_send(action) if block.call) : public_send(action)
        }
      end
    rescue ArgumentError
      nil
    end
  end
end
